% Matlab code example for Algorithms 4.4 and 4.5 presented in
% "Deterministic Sparse FFT for M-sparse Vectors"
% by Gerlind Plonka, Katrin Wannenwetsch, Annie Cuyt and Wen-shin Lee

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Implementation of Algorithm 4.5.
%
% [sigmaopt,distopt]=algorithm2(N,M,supp)
%
% This function returns for a given support the best prime "sigmaopt"
% among M primes < N/2.
% Aditionally, the function returns the minimal distance "distopt" 
% contained in \sigmaD.



function [sigmaopt]=algorithm2(N,M,supp)

if length(supp)==1 
    sigmaopt=1;
    return;
end

% vector with possible primes for sigma
sigmas=primes(N/2-1);
sigmas=sigmas(max(end-M+1,0):end);



% compute minimal distances for all possible sigmas
mindist=zeros(length(sigmas),1);

for s=1:length(sigmas)
    sigmasupp=zeros(length(supp),1);
    sigmasupp=sort(mod(sigmas(s)*supp,N));
    sigmasupp=[sigmasupp;sigmasupp(1)];
    sigmadist=zeros(length(supp),1);
    sigmadist(1:length(supp))=mod(sigmasupp(2:end)-sigmasupp(1:end-1),N);
    mindist(s)=min(sigmadist);
end

% find indices with maximal minimal distances
indopt=find(mindist==max(mindist));

if length(indopt)>1
    sumknots=zeros(length(indopt),1);
    for j=1:length(indopt)
        sumknots(j)=abs(sum(exp(-2*pi*1i*sigmas(indopt(j))*supp/N)));
    end
    indopt=indopt(find(sumknots==min(sumknots),1));
end

sigmaopt=sigmas(indopt);
end