% Matlab code example for Algorithms 2.3 presented in
% "Deterministic Sparse FFT for M-sparse Vectors"
% by Gerlind Plonka, Katrin Wannenwetsch, Annie Cuyt and Wen-shin Lee

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Implementation of Algorithm 2.3.
%
% [xreconstruction, activeindices, M] = algorithmshort(data)
%
% This function returns for a given data vector of length N=2^J the 
% inverse sparse FFT. The vector sparsity needs not to be known beforehand.
% The function uses algorithm1.m or algorithm2.m for evaluating the 
% parameter sigma.
% The function returns for x= IDFT(data)
% xreconstruction: the short vector containing only the significant
% components of x
% activeindices: the support indices of x 
% M=length(activeindices)= length(xreconstruction): sparsity of x

function [xreconstruction, activeindices, M] = algorithmshort(data)

% compute N and J
N = length(data);
J = log2(N);

% fix parameter for threshold
eps = 0.1;

% vector containing indices of nonzero components (= set "I" in algorithm)
activeindices = [];
M = 0; %M is the sparsity

% x^{(0)} = xhat(0)
xj = data(1); 

% if xhat(0) = 0, then x = 0
if abs(xj) < eps
    xreconstruction = zeros(N, 1);
    return;
end

% x contains at least one nonzero component
activeindices = 1;
M=1;


L=0.5;
sigma=0;
for j = 0:J - 1
        L=2*L; L1=N/L/2;
        if M^2 >= L/2
        % create vector zhat
        z1(1:L,1) = data(L1 * (2 * (1:L) - 1) + 1);
        %expand xj to length 2^j if it is given only as a short vector of
        %active indices
        if length(xj) ~= L 
            xj=zeros(L,1);
            xj(activeindices)=xj1;
        end
        % compute x^{(j+1)} using an FFT algorithm
        u = exp(pi * 1i * (0:L - 1) / L) .' .* ifft(z1);
        xj = 0.5 * [xj + u; xj - u];
        activeindices = find(xj > eps);
        xj1=xj(activeindices);
        
    else
        % compute sigma
        % if number of active indices has not changed in last step, just double sigma
        if (M == Mold && sigma >0)
            sigma = 2 * sigma;
        % otherwise compute sigma by algorithm 1 or 2
        else
           sigma = algorithm2(L, M, activeindices);
        end
        
        % create matrix V_M
        v1=exp(-2*pi*1i*sigma*(activeindices(1:M)-1)/L)';
        V=ShortVander(v1,M);
        V=V'*diag(exp(-pi*1i*(activeindices(1:M)-1)/L)); 
        
        % create vector zhat
        z2 = zeros(M,1);
        z2(1:M,1)=data(L1*(2* mod((0:M-1)*sigma,L)+1)+1);
        % compute x^{(j+1)} using the Vandermonde matrix
        v=(xj1(1:M) + (V\z2))./2;
        
        % compute xj
        activeindices1 = find(v > eps);
        xs=xj1-v;
        activeindices2= find(xs > eps);
        
        xj1=[v(activeindices1); xs(activeindices2)];
        activeindices=[activeindices(activeindices1);activeindices(activeindices2)+2^j];
    end
    
    % determine active indices and sparsity M for vector xj
    Mold=M;
    M = length(activeindices);
    
    
end

xreconstruction = xj1;
activeindices = activeindices - 1;

end

